/*
 *  Copyright (C) 2002 Tomasz Kojm <zolw@konarski.edu.pl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "readdb.h"
#include "others.h"
#include "strings.h"
#include "shared.h"
#include "matcher.h"

int parse_add(struct node *root, const char *virname, const char *hexstr)
{
	struct patt *new;

    /* decode hexstring and prepare new entry */

    new = (struct patt *) mcalloc(1, sizeof(struct patt));
    new->length = strlen(hexstr) / 2;

    if(new->length > root->maxpatlen)
	root->maxpatlen = new->length;

    if((new->pattern = hex2str(hexstr)) == NULL) {
	free(new);
	return 1;
    }

    new->virname = mcalloc(strlen(virname) + 1, sizeof(char));
    strncpy(new->virname, virname, strlen(virname));

    addpatt(root, new);

    return 0;
}

/* this functions returns a pointer to the root of trie */

struct node *readdb(const char *filename, struct node *root)
{
	FILE *fd;
	char buffer[BUFFSIZE], *pt, *start;
	int line = 0;
	struct node *trie;

    if((fd = fopen(filename, "rb")) == NULL) {
	mprintf("!Can't open file %s\n", filename);
	return NULL;
    }

    mprintf("*Loading the database from %s\n", filename);

    if(!root) {
	trie = (struct node *) mcalloc(1, sizeof(struct node));
	trie->maxpatlen = 0;
    } else
	trie = root;

    while(fgets(buffer, BUFFSIZE, fd)) {
	line++;
	chomp(buffer);

	pt = strchr(buffer, '=');
	if(!pt) {
	    mprintf("!Malformed pattern line %d (file %s).\n", line, filename);
	    return NULL;
	}

	start = buffer;
	*pt++ = 0;

	if(*pt == '=') continue;

	claminfo.signs++;
	if(parse_add(trie, start, pt)) return NULL;
    }

    fclose(fd);
    return trie;
}

char *listdb(const char *dirname)
{
	static DIR *dd = NULL;
	struct dirent *dent;
	char *dbfile;

    if(!dd) {
	if((dd = opendir(dirname)) == NULL) {
	    /* this error shouldn't occur, because viruses.db is read first */
	    mprintf("@Can't open directory %s\n", dirname);
	    return NULL;
	}
    }


    while((dent = readdir(dd))) {
	if(dent->d_ino) {
	    if(strcmp(dent->d_name, ".") && strcmp(dent->d_name, "..") && strcmp(dent->d_name, "viruses.db") && strbcasestr(dent->d_name, ".db")) {
		dbfile = (char *) mcalloc(strlen(dent->d_name) + strlen(dirname) + 2, sizeof(char));
		sprintf(dbfile, "%s/%s", dirname, dent->d_name);
		return dbfile;
	    } else continue;
	}
    }

    closedir(dd);
    return NULL;
}
